#ifdef LCD

// LCD connector;
// 1   2   3   4   5   6   7   8   9   
// GND U+  A0  E   D4  D5  D6  D7  UBL+

//--- LCD FUNCTIONS ------------------------------------------------------

#define LCD_DELAY_BASE_100_US 10

#define DEBUG_DELAY
//#define DEBUG_DELAY Delay10ms(10);

void lcdSend4 (char c) {
  LCD_D4 = c & 0b0001 ? 1 : 0;
  LCD_D5 = c & 0b0010 ? 1 : 0;
  LCD_D6 = c & 0b0100 ? 1 : 0;
  LCD_D7 = c & 0b1000 ? 1 : 0;

  LCD_E = 1; 
  NOP(); NOP(); NOP(); NOP(); 
  DEBUG_DELAY; 
  LCD_E = 0;

  Delay100us (LCD_DELAY_BASE_100_US);
  DEBUG_DELAY;
}

void lcdSend(char c) {
  // high data byte;
  LCD_D4 = c & 0b00010000 ? 1 : 0;
  LCD_D5 = c & 0b00100000 ? 1 : 0;
  LCD_D6 = c & 0b01000000 ? 1 : 0;
  LCD_D7 = c & 0b10000000 ? 1 : 0;

  LCD_E = 1; 
  NOP(); NOP(); NOP(); NOP(); 
  DEBUG_DELAY; 
  LCD_E = 0;

  Delay100us (LCD_DELAY_BASE_100_US);
  DEBUG_DELAY;
  // low data byte;
  lcdSend4 (c);
}

void lcdWrite (char c) {
  LCD_A0 = 1;
  lcdSend (c);
}

void lcdCommand (char c) {
  LCD_A0 = 0;
  lcdSend (c);
}

void lcdCommand8 (char c) {
  lcdSend4 (c);
}

#define LCD_CMD_POS1		( 0x80 )
#define LCD_CMD_POS2		(0x80 | 0x40)

#define LCD_CURSOR_ON		lcdCommand (0xC | 0x2);
#define LCD_CURSOR_OFF		lcdCommand (0xC);
#define LCD_CURSOR_BLINK_ON	lcdCommand (0xC | 0x2 | 0x1);
#define LCD_CLEAR		{ lcdCommand (0x1); Delay100us (15+LCD_DELAY_BASE_100_US); }
#define LCD_CURSOR_HOME		lcdCommand (0x2);
#define LCD_SHIFT_CURSOR_LEFT	lcdCommand (0x10);
#define LCD_SHIFT_CURSOR_RIGHT	lcdCommand (0x10 | 0x4);
#define LCD_SHIFT_DISPLAY_LEFT	lcdCommand (0x10 | 0x8);
#define LCD_SHIFT_DISPLAY_RIGHT	lcdCommand (0x10 | 0x8 | 0x4);
#define LCD_CURSOR_LINE_1(x)	lcdCommand (LCD_CMD_POS1 | x);
#define LCD_CURSOR_LINE_2(x)	lcdCommand (LCD_CMD_POS2 | x);

void lcdProgram (const char * p) {
  byte i;
  char c;
  char lbCommand = 1;
  for (i = 0; ; i++) {
    char c = p[i];
    if (c == 0)
      return;
    if (lbCommand) {
      lcdCommand (c);
      if (c & LCD_CMD_POS1) // output character after cursor placement command;
        lbCommand = 0;
    } else {
      lcdWrite (c);
      lbCommand = 1;
    }
  }
}

void lcdWriteln (const char * p) {
  byte i;
  for (i = 0; p[i] != 0; i++) {
      lcdWrite(p[i]);
  }
}

const char LCD_Data_Init[] = {
  0x28, //  
  0x0C, //  
  0x6,  //     
  0x0F, //     
//  0x80, //  
//  ' ',
  0x01,  // ,  
  0
};

void LCD_Init(void) { // 16s2d
  LCD_E = 0;
  LCD_A0 = 0;
  Delay10ms (5);
  Delay10ms (5);
  Delay10ms (5);
  Delay10ms (5);
  Delay10ms (5);

// 30h  
  lcdCommand8 (3);
  lcdCommand8 (3);
  lcdCommand8 (3);

// 20h  
  lcdCommand8 (2);

//4bit
  lcdProgram (LCD_Data_Init);
  Delay100us (15);
  Delay100us (15);
  Delay100us (15);
  Delay100us (15);
  Delay100us (15);
  Delay100us (15);
  Delay100us (15);
  Delay100us (15);
  Delay100us (15);
  Delay100us (15);
}

void WriteNum (uint16 n) {
  byte i;
  i = '0';
  while (n >= 1000) {
    n -= 1000;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 100) {
    n -= 100;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 10) {
    n -= 10;
    i++;
  }
  lcdWrite (i);
  lcdWrite (n+'0');
}

void WriteLNum (uint16 n) {
  byte i;
  i = '0';
  while (n >= 10000) {
    n -= 10000;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 1000) {
    n -= 1000;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 100) {
    n -= 100;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 10) {
    n -= 10;
    i++;
  }
  lcdWrite (i);
  lcdWrite (n+'0');
}

void WriteBNum (byte n) {
  byte i;
  i = '0';
  while (n >= 100) {
    n -= 100;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 10) {
    n -= 10;
    i++;
  }
  lcdWrite (i);
  lcdWrite (n+'0');
}

void WriteSNum (int16 n) {
  byte i;
  if (n < 0) {
    lcdWrite ('-');
    n = -n;    
  } else {
    lcdWrite ('+');
  }

  i = '0';
  while (n >= 1000) {
    n -= 1000;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 100) {
    n -= 100;
    i++;
  }
  lcdWrite (i);

  i = '0';
  while (n >= 10) {
    n -= 10;
    i++;
  }
  lcdWrite (i);
  lcdWrite (n+'0');
}

void Write1Digit(byte n) {
  lcdWrite (n+'0');
}

#endif

