#include <pic18.h>

	__CONFIG(1, OSCSEN & XT);
	__CONFIG(2, BOREN & BORV27 & PWRTEN & WDTEN & WDTPS128);
	__CONFIG(3, CCP2RB3);
	__CONFIG(4, DEBUGDIS & LVPDIS & STVREN);
	__CONFIG(5, UNPROTECT);
	__CONFIG(6, WRTEN);
	__CONFIG(7, TRU);

#define LCD

#include "types.h"

#include "delay8m.c"

#define  TrisA 0b00101111
//               ||||||||
//               |||||||POT0
//               ||||||POT1
//               |||||POT2
//               ||||POT3
//               |||
//               ||POT4
//               |
//                 

#define  TrisB 0b00000000
//               ||||||||
//               |||||||SERVO0
//               ||||||SERVO1
//               |||||SERVO2
//               ||||SERVO3
//               |||SERVO4
//               ||PPM
//               |PPM_INV
//                 

#define  TrisC 0b00000000
//               ||||||||
//               |||||||A0
//               ||||||LED
//               |||||
//               ||||D0
//               |||D1
//               ||D2
//               |D3 
//               E  

#ifdef LCD
// LCD connector;
// 1   2   3   4   5   6   7   8   9   
// GND U+  A0  E   D4  D5  D6  D7  UBL+
#define	LCD_A0    LC0   // LCD-4  - PIC-11
#define	LCD_D4    LC3   // LCD-11 - PIC-14
#define	LCD_D5    LC4   // LCD-12 - PIC-15
#define	LCD_D6    LC5   // LCD-13 - PIC-16
#define	LCD_D7    LC6   // LCD-14 - PIC-17
#define	LCD_E     LC7   // LCD-6  - PIC-18
#endif

#include "lcd8m.c"

#define SERVO_0 LB0
#define SERVO_1 LB1
#define SERVO_2 LB2
#define SERVO_3 LB3
#define SERVO_4 LB4
#define PPM     LB5
#define PPM_INV LB6

#define NUM_SERVOS 5

static int16 servo[NUM_SERVOS];
static byte curServo;
static bit ppm_gap;

#define PPM_GAP 250

#if NUM_SERVOS < 1
#error NUM_SERVOS has to be greater than 0
#endif

#if NUM_SERVOS > 5
#error NUM_SERVOS has to be less or equal to 5
#endif

// Set servo to 0 or 1;
// It is not supported to pass out_0_1 as bit;
void setServo(byte servo, byte out_0_1) {
	switch(servo*2+out_0_1) {
case 0*2+0: SERVO_0 = 0; break;
case 0*2+1: SERVO_0 = 1; break;
#if NUM_SERVOS > 1
case 1*2+0: SERVO_1 = 0; break;
case 1*2+1: SERVO_1 = 1; break;
#if NUM_SERVOS > 2
case 2*2+0: SERVO_2 = 0; break;
case 2*2+1: SERVO_2 = 1; break;
#if NUM_SERVOS > 3
case 3*2+0: SERVO_3 = 0; break;
case 3*2+1: SERVO_3 = 1; break;
#if NUM_SERVOS > 4
case 4*2+0: SERVO_4 = 0; break;
case 4*2+1: SERVO_4 = 1; break;
#endif
#endif
#endif
#endif  
	}
}

// Timing diagramm
//    0     1      2
// ___/-----\____________________________
// _________/------\_____________________

// by default it is high priority interrupt vector;
static void interrupt
isr(void)			// Here be interrupt function - the name is unimportant.
{
     if (TMR0IF) { // ----------- TMR0_Interrupt 

	if (!ppm_gap) { // Starting from PPM gap;
		PPM = 1;
		PPM_INV = 0;

		if (curServo > 0)
			setServo(curServo-1, 0); // Fall previous servo;

		if (curServo < NUM_SERVOS)
			setServo(curServo, 1); // Raise curServo;

		WRITETIMER0 (65536-PPM_GAP);
		ppm_gap = 1;
	} else {
		PPM = 0;
		PPM_INV = 1;
		ppm_gap = 0;

		if (curServo == NUM_SERVOS) {
			WRITETIMER0 (65536-40000-PPM_GAP); // TMR0 runs at 1MHz, 1 tick=1us. 40000 ticks = 40ms= 25Hz;
			curServo = 0;
		} else {
			WRITETIMER0 ((65536-1500+77+PPM_GAP)-servo[curServo]); // 1 ms + servo;
			curServo++; // Step to next servo;
		}
	}

	TMR0IF = 0;         
     }

}

void delay4us() { // 2*0.5 (call)+(2*0.5 ret)+1(nop) = 5us;
	unsigned char _dcnt = 5;
	while(--_dcnt > 0)
		NOP(); 
}

void main(void) {
	PORTA = PORTB = PORTC = 0;
	LATA = LATB = LATC = 0;
	TRISA = TrisA; 
	TRISC = TrisC;
	TRISB = TrisB;

	OSCCON = 0b00001000; // 8 MHz;

	ADCON1 = 0b10000010; // Right justified, bit6=Fosc/32, AN0-4;
	ADCON0 = 0b10000001; // bit7,6=Fosc/32, AN0, GO=0, AD used; 

	T0CON = 0b00000000; // Works with Fosc/4 = 8MHz/4=2MHz, 16 bit, int, prescaler 1 to 2;

	IPEN = 1; // Enable priority for interrupts;
	PEIE = 1; // Enable peripherial interrupts;
	TMR0IP = 1; // Timer0 interrupt is high priority interrupt;
	TMR0IF = 0; // Clear any pending Timer0 interrupts;

	TMR0IE = 1;
	GIEH = 1; // Enable high priority interrupts;
	GIEL = 1; // Enable low priority interrupts;
	GIE = 1;

#ifdef LCD
	LCD_Init();
	LCD_CURSOR_OFF;
/*
	LCD_CURSOR_LINE_1 (1);
	lcdWriteln ( "Servo preseter v1.0");
	Delay10ms(10);
	LCD_CURSOR_LINE_2 (0);
	lcdWriteln ("(c) Andrey Prikupets");
	Delay10ms(10);
	LCD_CLEAR;
*/
#endif

	// Reset channels;
	for (curServo=0; curServo<NUM_SERVOS; curServo++) {
		servo[curServo] = 0;
	}
	curServo = 0;
	ppm_gap = 0;

	WRITETIMER0 (0); 

	TMR0ON = 1; // Start timer;

	while (1) {
		Delay10ms(2);
		for (byte channel=0; channel<NUM_SERVOS; channel++) {
			int16 value;

			ADCON0 = ADCON0 & ~0b00111000 | (channel << 3);
			delay4us(); // Wait 2*Tad = 2*1.6us = 3.2us; 
//			Delay100us(1);

			GODONE = 1;
			while (GODONE) { }; // (ADC needs 12*TAD for conversion);

			value = ((((uint16)ADRESH) << 8) | ADRESL)-512;
#ifdef LCD
			switch (channel) {
		case 0: { LCD_CURSOR_LINE_1(0); WriteSNum (value); break; }
#if NUM_SERVOS > 1
		case 1: { LCD_CURSOR_LINE_1(5); WriteSNum (value); break; }
#if NUM_SERVOS > 2
		case 2: { LCD_CURSOR_LINE_1(10); WriteSNum (value); break; }
#if NUM_SERVOS > 3
		case 3: { LCD_CURSOR_LINE_2(0); WriteSNum (value); break; }
#if NUM_SERVOS > 4
		case 4: { LCD_CURSOR_LINE_2(5); WriteSNum (value); break; }
#endif
#endif
#endif
#endif  
			}
#endif 
			servo[channel] = value;
			CLRWDT(); 
		}
	}
}

